{% extends "admin/change_form.html" %}
{% load i18n %}
{% load loginas_javascript %}

{% block object-tools-items %}
    {{ block.super }}
    <li><a id="loginas-link" href="#"><i class="icon-user icon-alpha75"></i>{% trans "Log in as user" %}</a></li>

{% endblock %}

{% block footer %}
    {{ block.super }}
    <form method="post" id="loginas-form" action="{% url 'loginas-user-login' object_id %}" style="display: none">
        {% csrf_token %}
        <input type="hidden" name="reason" id="loginas-reason-input" value="">
    </form>

    {% modal %}
    {% javascript %}
{% endblock %}
